@@ -455,12 +455,11 @@ int win32_net_http_open(char* url, struct httpdata *hd)
 
 		mpg123_add_string(&request_url, purl.p);
 
+		if(!split_url(&purl, NULL, &host, &port, &path)){ oom=1; goto exit; }
 		if (hd->proxystate >= PROXY_HOST)
 		{
 			/* We will connect to proxy, full URL goes into the request. */
-			if(    !mpg123_copy_string(&hd->proxyhost, &host)
-			    || !mpg123_copy_string(&hd->proxyport, &port)
-			    || !mpg123_set_string(&request, "GET ")
+			if(    !mpg123_set_string(&request, "GET ")
 			    || !mpg123_add_string(&request, request_url.p) )
 			{
 				oom=1; goto exit;
@@ -469,7 +468,6 @@ int win32_net_http_open(char* url, struct httpdata *hd)
 		else
 		{
 			/* We will connect to the host from the URL and only the path goes into the request. */
-			if(!split_url(&purl, NULL, &host, &port, &path)){ oom=1; goto exit; }
 			if(    !mpg123_set_string(&request, "GET ")
 			    || !mpg123_add_string(&request, path.p) )
 			{
@@ -480,6 +478,14 @@ int win32_net_http_open(char* url, struct httpdata *hd)
 		if(!fill_request(&request, &host, &port, &httpauth1, &try_without_port)){ oom=1; goto exit; }
 
 		httpauth1.fill = 0; /* We use the auth data from the URL only once. */
+		if (hd->proxystate >= PROXY_HOST)
+		{
+			if(    !mpg123_copy_string(&hd->proxyhost, &host)
+			    || !mpg123_copy_string(&hd->proxyport, &port) )
+			{
+				oom=1; goto exit;
+			}
+		}
 		debug2("attempting to open_connection to %s:%s", host.p, port.p);
 		win32_net_open_connection(&host, &port);
 		if(ws.local_socket == SOCKET_ERROR)
